//MyWinFile.cpp
#include <assert.h>
#include <time.h>
#include "MyFileType.h"
#include "MyWinFile.h"


//-----------------------------------------------------------------------------
MyWinFile::MyWinFile()
: m_hFile(INVALID_HANDLE_VALUE)
{
}
//-----------------------------------------------------------------------------
MyWinFile::~MyWinFile()
{
	this->Close();
}
//-----------------------------------------------------------------------------
bool MyWinFile::Open(LPCTSTR szFileName)
{
	return this->Create(szFileName);
}
//-----------------------------------------------------------------------------
bool MyWinFile::Close()
{
	if(m_hFile != INVALID_HANDLE_VALUE)   
	{
		::CloseHandle(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
		m_fileName.clear();
	}
	return true;
}
//-----------------------------------------------------------------------------
bool MyWinFile::Create(LPCTSTR szFileName)
{
	m_hFile = ::CreateFile(
		szFileName,
		GENERIC_READ | GENERIC_WRITE, 
		FILE_SHARE_READ | FILE_SHARE_WRITE, 
		NULL,
        OPEN_EXISTING, 
		0,
        0);

	if(m_hFile != INVALID_HANDLE_VALUE)
		m_fileName = szFileName;

	return (m_hFile != INVALID_HANDLE_VALUE);
}
//-----------------------------------------------------------------------------
DWORD MyWinFile::Read(void* pBuffer,DWORD numBytes)
{
	assert(this);

	DWORD numRead = 0;   
	::ReadFile(m_hFile, pBuffer, numBytes, &numRead, 0);   
	return numRead; 
}
//-----------------------------------------------------------------------------
DWORD MyWinFile::Write(void* pBuffer,DWORD numBytes)
{
	assert(this);

	DWORD numWritten = 0;   
	::WriteFile(m_hFile, pBuffer, numBytes, &numWritten, 0);   
	return numWritten;
}
//-----------------------------------------------------------------------------
bool MyWinFile::GetModifiedTime(TimeInfo& out)
{
	FILETIME writtenTime;
	BOOL bResult = ::GetFileTime(m_hFile, 0,0, &writtenTime);
	if( bResult == FALSE )
		return false;

	SYSTEMTIME systemTime, localTime;

	//˱⽬ SYSTEMTIME ȯ
	::FileTimeToSystemTime(&writtenTime, &systemTime);

	//UTC ð ð ȯ
	::SystemTimeToTzSpecificLocalTime(NULL, &systemTime, &localTime);

	memset(&out, 0, sizeof(TimeInfo));
	out.year = (int)localTime.wYear;
	out.month = (int)localTime.wMonth;
	out.day = (int)localTime.wDay;
	out.hour = (int)localTime.wHour;
	out.min = (int)localTime.wMinute;
	out.sec = (int)localTime.wSecond;
	out.milliSec = (int)localTime.wMilliseconds;
	
	return true;
}
//-----------------------------------------------------------------------------
bool MyWinFile::Exists()
{
	return (m_hFile != INVALID_HANDLE_VALUE);
}
//-----------------------------------------------------------------------------
LONGLONG MyWinFile::GetSize()
{
	LARGE_INTEGER liSize;
	liSize.QuadPart = 0;
	BOOL res = ::GetFileSizeEx(m_hFile, &liSize);   
	
	if(res == FALSE)
		return 0;
	return liSize.QuadPart;
}
//-----------------------------------------------------------------------------
void MyWinFile::FlushBuffers()
{
	if(m_hFile != INVALID_HANDLE_VALUE)
		::FlushFileBuffers(m_hFile);
}
//-----------------------------------------------------------------------------
__int64 MyWinFile::GetPos()
{
	LARGE_INTEGER distance, newPos;
	distance.QuadPart = 0;
	newPos.QuadPart = 0;

	BOOL res = SetFilePointerEx(m_hFile, distance, &newPos, FILE_CURRENT);
	
	if(res == TRUE)
		return newPos.QuadPart;
	else
		return -1;
}
//-----------------------------------------------------------------------------
bool MyWinFile::Seek(__int64 offset)
{
	LARGE_INTEGER distance, newPos;
	distance.QuadPart = offset;
	newPos.QuadPart = 0;

	BOOL res = SetFilePointerEx(m_hFile, distance, &newPos, FILE_BEGIN);
	
	return (res == TRUE);
}
//-----------------------------------------------------------------------------
void MyWinFile::SeekEnd()
{
	::SetFilePointer(m_hFile, 0, 0, FILE_END);
}